%%%%%%%%%
% CLEAR %
%%%%%%%%%

% Close all windows
close all;

% Clear all variables
clear all;

% Clear command window
clc;

%%%%%%%%%%%%%%
% USER INPUT %
%%%%%%%%%%%%%%

% User input
patient = input ( 'Enter the patient number [ 1 / 2 / 3 / 4 ]: ' );

%%%%%%%%%%%%%%%%%%%%
% GET PATIENT DATA %
%%%%%%%%%%%%%%%%%%%%

% Get patient data
[ a , b , g , n ] = get_patient_data ( patient );

%%%%%%%%%%%%%%%%%
% SYSTEM MATRIX %
%%%%%%%%%%%%%%%%%

% System matrix definition
A = [ - a , + b ; - g , - n ];

%%%%%%%%%%%%%%%%
% INPUT MATRIX %
%%%%%%%%%%%%%%%%

% Input matrix definition
B = [ 1 ; 1 ];

%%%%%%%%%%%%%%%%%
% OUTPUT MATRIX %
%%%%%%%%%%%%%%%%%

% Output matrix definition
C = [ 0 , 0 ; 0 , 1 ];

%%%%%%%%%%%%%%%%%%%%%%
% FEEDTRHOUGH MATRIX %
%%%%%%%%%%%%%%%%%%%%%%

% Feedtrough matrix definition
D = [ 0 ; 0 ];

%%%%%%%%%%%%%%%%%%%%%
% STATE SPACE MODEL %
%%%%%%%%%%%%%%%%%%%%%

% State space model
Systm = ss ( A , B , C , D );

%%%%%%%%%%%%%%%%%%%%%%%
% EXTRACT % ZERO POLE %
%%%%%%%%%%%%%%%%%%%%%%%

% Extract zero pole parameters
[ z , p , k ] = zpkdata ( Systm );

%%%%%%%%%%%%%%%%%%%%%%%%
% RUN % SIMULATION 001 %
%%%%%%%%%%%%%%%%%%%%%%%%

% Time vector definition
t   = 0:0.01:10;

% Input vector
U   = zeros ( size ( t ) );

% Input variables
x01 = [ 0.01 ; 1 ];

%%%%%%%%%%%%%%%%%%%%%%%%
% OPEN LOOP % RESPONSE %
%%%%%%%%%%%%%%%%%%%%%%%%

% Open loop response
[ Yo , t , Xo ] = lsim ( Systm , U , t , x01 );

%%%%%%%%%%%%%%%%%%%%%%%%%
% FIGURE INITIALIZATION %
%%%%%%%%%%%%%%%%%%%%%%%%%

% Figure
figure;

%%%%%%%%%%%%%%%%%%%%%%%%
% SUBPLOT % DEFINITION %
%%%%%%%%%%%%%%%%%%%%%%%%

% Subplot position
subplot ( 211 );

%%%%%%%%
% PLOT %
%%%%%%%%

% Plot response
plot ( t , Xo );

%%%%%%%%%%%%%%%%%%
% PLOT % OPTIONS %
%%%%%%%%%%%%%%%%%%

% Title definition
title  ( strcat ( 'State sapce parameters along sleep time - Patient ' , num2str ( patient ) ) );

% X axis lable
xlabel ( 'Sleep time [ h ]' );

% Y axis lable
ylabel ( 'Amplitude' );

%%%%%%%%%%%%%%%%%%%%%%%%
% SUBPLOT % DEFINITION %
%%%%%%%%%%%%%%%%%%%%%%%%

% Subplot position
subplot ( 212 );

%%%%%%%%
% PLOT %
%%%%%%%%

% Plot response
plot ( t , Yo ( : , 2 ) );

%%%%%%%%%%%%%%%%%%
% PLOT % OPTIONS %
%%%%%%%%%%%%%%%%%%

% Title definition
title  ( strcat ( 'Glucose level along sleep time - Patient ' , num2str ( patient ) ) );

% X axis lable
xlabel ( 'Sleep time [ h ]' );

% Y axis lable
ylabel ( 'Glucose level' );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EIGENVALUE % CALCULATION %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Eigenvalue calculation
eig_val = eig ( A );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK % STABILITY % STABLE %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Check for system stability
if ( real ( eig_val ( 1 , 1 ) ) < 0 ) && ( real ( eig_val ( 2 , 1 ) ) < 0 ) 

    %%%%%%%%%%%%%%%%%
    % SYSTEM STABLE %
    %%%%%%%%%%%%%%%%%
    
    % System stable
    disp ( 'System is Stable' );
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK % STABILITY % NOT STABLE %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Check for system stability    
else
    
    %%%%%%%%%%%%%%%%%%%%%
    % SYSTEM NOT STABLE %
    %%%%%%%%%%%%%%%%%%%%%
    
    % System not stable
    disp ( 'System is NOT Stable' );
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%
% CONTROLLABILITY MATRIX %
%%%%%%%%%%%%%%%%%%%%%%%%%%

% Controllability matrix definition
P = ctrb ( Systm );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK % CONTROLLABILITY % TRUE %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Check for system controllability
if ( rank ( P ) == size ( A , 1 ) )
    
    %%%%%%%%%%%%%%%%%%%%%%%
    % SYSTEM CONTROLLABLE %
    %%%%%%%%%%%%%%%%%%%%%%%
    
    % System controllable
    disp ( 'System is Controllable' );
   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK % CONTROLLABILITY % FALSE %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Check for system controllability
else
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%
    % SYSTEM NOT CONTROLLABLE %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % System not controllable
    disp ( 'System is NOT Controllable' );
    
end

%%%%%%%%%%%%%%%%%%%%%%%%
% OBSERVABILITY MATRIX %
%%%%%%%%%%%%%%%%%%%%%%%%

% Observability matrix definition
Q = obsv ( Systm );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK % OBSERVABILITY % TRUE %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Check for system observability
if ( rank ( Q ) == size ( A , 1 ) )
    
    %%%%%%%%%%%%%%%%%%%%%
    % SYSTEM OBSERVABLE %
    %%%%%%%%%%%%%%%%%%%%%
    
    % System observable
    disp ( 'System is Observable' );
   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK % OBSERVABILITY % FALSE %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Check for system observability
else
    
    %%%%%%%%%%%%%%%%%%%%%%%%%
    % SYSTEM NOT OBSERVABLE %
    %%%%%%%%%%%%%%%%%%%%%%%%%
    
    % System not observable
    disp ( 'System is NOT Observable' );
    
end

%%%%%%%%%%%%%%%%%%%
% GAIN DEFINITION %
%%%%%%%%%%%%%%%%%%%

% New desired eigenvalue
newEig = z { 2 };

% Gain K definition
desrd_EigVal = [ 0 newEig ];

%%%%%%%%%%%%%%%%%%%%%
% CONTROLLER MATRIX %
%%%%%%%%%%%%%%%%%%%%%

% Controller matrix definition
K = place ( A , B , desrd_EigVal );

%%%%%%%%%%%%%%%%%%%%%%%%%%
% UPDATE % SYSTEM MATRIX %
%%%%%%%%%%%%%%%%%%%%%%%%%%

% Update system matrin
Ac = A - ( B * K );

%%%%%%%%%%%%%%%%%%%%%%%%%
% UPDATE % INPUT MATRIX %
%%%%%%%%%%%%%%%%%%%%%%%%%

% Update input matrix
Bc = B;

%%%%%%%%%%%%%%%%%%%%%%%%%%
% UPDATE % OUTPUT MATRIX %
%%%%%%%%%%%%%%%%%%%%%%%%%%

% Update output matrix
Cc = C;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% UPDATE % FEEDTRHOUGH MATRIX %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Update feedtrough matrix
Dc = D;

%%%%%%%%%%%%%%%%%%%%%
% STATE SPACE MODEL %
%%%%%%%%%%%%%%%%%%%%%

% State space model
systmc = ss ( Ac , Bc , Cc , Dc );

%%%%%%%%%%%%%%%%%%%%%%%%%
% CLOSE LOOP % RESPONSE %
%%%%%%%%%%%%%%%%%%%%%%%%%

% Close loop response
[ Yc , t , Xc ] = lsim ( systmc , U , t , x01 );

%%%%%%%%%%%%%%%%%%%%%%%%%
% FIGURE INITIALIZATION %
%%%%%%%%%%%%%%%%%%%%%%%%%

% Figure
figure;

%%%%%%%%%%%%%%%%%%%%%%%%
% SUBPLOT % DEFINITION %
%%%%%%%%%%%%%%%%%%%%%%%%

% Subplot position
subplot ( 211 );

%%%%%%%%
% PLOT %
%%%%%%%%

% Plot response
plot ( t , Xc );

%%%%%%%%%%%%%%%%%%
% PLOT % OPTIONS %
%%%%%%%%%%%%%%%%%%

% Title definition
title  ( strcat ( 'State sapce parameters along sleep time - Patient ' , num2str ( patient ) ) );

% X axis lable
xlabel ( 'Sleep time [ h ]' );

% Y axis lable
ylabel ( 'Amplitude' );

%%%%%%%%%%%%%%%%%%%%%%%%
% SUBPLOT % DEFINITION %
%%%%%%%%%%%%%%%%%%%%%%%%

% Subplot position
subplot ( 212 );

%%%%%%%%
% PLOT %
%%%%%%%%

% Plot response
plot ( t , Yc ( : , 2 ) );

%%%%%%%%%%%%%%%%%%
% PLOT % OPTIONS %
%%%%%%%%%%%%%%%%%%

% Title definition
title  ( strcat ( 'Glucose level along sleep time - Patient ' , num2str ( patient ) ) );

% X axis lable
xlabel ( 'Sleep time [ h ]' );

% Y axis lable
ylabel ( 'Glucose level' );

%%%%%%%%%%
% FIGURE %
%%%%%%%%%%

% Figure initialization
figure;

%%%%%%%%%%%%%%%%%%%%%%%%
% SUBPLOT % DEFINITION %
%%%%%%%%%%%%%%%%%%%%%%%%

% Subplot position
subplot ( 211 );

%%%%%%%%
% PLOT %
%%%%%%%%

% Plot response
plot ( t , Xo ( : , 1 ) , 'r' , t , Xc ( : , 1 ) , 'b' );

%%%%%%%%%%%%%%%%
% PLOT OPTIONS %
%%%%%%%%%%%%%%%%

% Grid on
grid on;

% Legend
legend ( 'Open-loop' , 'Closed-loop' );

% Y axis label
ylabel ( 'Blood insulin level' );

%%%%%%%%%%%%%%%%%%%%%%%%
% SUBPLOT % DEFINITION %
%%%%%%%%%%%%%%%%%%%%%%%%

% Subplot position
subplot ( 212 );

%%%%%%%%
% PLOT %
%%%%%%%%

% Plot response
plot ( t , Xo ( : , 2 ) , 'r' , t , Xc ( : , 2 ) , 'b' );

%%%%%%%%%%%%%%%%
% PLOT OPTIONS %
%%%%%%%%%%%%%%%%

% Grid on
grid on;

% Legend
legend ( 'Open-loop' , 'Closed-loop' );

% Y axis label
ylabel ( 'Glucose concentration' );
